# coding: utf-8
"""Логгер действий пользователя в системе.

.. moduleauthor:: Andrey Torsunov <torsunov@bars-open.ru>

"""

__author__ = 'Andrey Torsunov'
__contact__ = 'torsunov@bars-open.ru'
__docformat__ = 'restructuredtext'
